/* This file is in the public domain */

#include <string>

#include <opencl/filters.h>
#include <opencl/lookup.h>

#include <opencl/emac.h>
#include <opencl/hmac.h>
using namespace OpenCL;

Filter* lookup_mac(const std::string& algname, const SymmetricKey& key)
   {
   Filter* mac = 0;
   try {
     mac = new MAC_Filter(algname, key);
   }
   catch(Algorithm_Not_Found) {}

   if(mac)
      return mac;

   // These are all cases which *should* be handled by lookup
   std::vector<std::string> name = parse_algorithm_name(algname);
   if(name.size() != 2)
      return 0;

   const std::string mac_name = name[0];
   const std::string hash_name = name[1];

   if(mac_name == "EMAC")
      return new MAC_Filter(new EMAC(hash_name), key);
   if(mac_name == "HMAC")
      return new MAC_Filter(new HMAC(hash_name), key);
   return 0;
   }
