/* This file is in the public domain */

#include <string>
#include <cstdlib>
#include <ctime>

#include <opencl/filters.h>

#include <opencl/randpool.h>
#include <opencl/x917_rng.h>

using namespace OpenCL;

/* Not too useful generally; just dumps random bits for benchmarking */
class RNG_Filter : public Filter
   {
   public:
      void write(const byte[], u32bit);
      RNG_Filter(RandomNumberGenerator* r) : rng(r), buffer(1024)
         { position = 0; }
      ~RNG_Filter() { delete rng; }
   private:
      RandomNumberGenerator* rng;
      SecureVector<byte> buffer;
      u32bit position;
   };

void RNG_Filter::write(const byte[], u32bit length)
   {
   while(length)
      {
      u32bit gen = std::min(buffer.size() - position, length);
      rng->randomize(buffer, buffer.size());
      length -= gen;
      }
   }

Filter* lookup_rng(const std::string& algname)
   {
   if(algname == "X9.17(Rijndael)")
      return new RNG_Filter(new ANSI_X917_RNG);
   else if(algname == "Randpool")
      return new RNG_Filter(new Randpool);
   else return 0;
   }
