/*
An OpenCL example application which emulates a poorly written version of GNU's
md5sum (though it is significantly faster than md5sum, at least on my
machine). It's output is also compatible with md5sum; you can check hashes
generated by this program with md5sum.

Back in the GCC 2.95.x/3.0.x days, this version was a lot (~2.5 times) slower
than the md5_fd program, which is just like this one, except it uses the Unix
I/O system calls. Things have evened out with GCC 3.1

Written by Jack Lloyd (lloyd@acm.jhu.edu), on date or dates unknown

This file is in the public domain
*/

#include <iostream>
#include <fstream>
#include <opencl/md5.h>
#include <opencl/hex.h>
#include <opencl/filters.h>

int main(int argc, char* argv[])
   {
   if(argc < 2)
      {
      std::cout << "Usage: md5 <filenames>" << std::endl;
      return 1;
      }
   try {

   OpenCL::LibraryInitializer init;

   OpenCL::Pipe pipe(new OpenCL::Hash_Filter(new OpenCL::MD5),
                     new OpenCL::Hex_Encoder);

   int skipped = 0;
   for(int j = 1; argv[j] != 0; j++)
      {
      std::ifstream file(argv[j]);
      if(!file)
         {
         std::cout << "ERROR: could not open " << argv[j] << std::endl;
         skipped++;
         continue;
         }
      pipe.start_msg();
      file >> pipe;
      pipe.end_msg();
      file.close();
      pipe.set_default_msg(j-1-skipped);
      std::cout << pipe << "  " << argv[j] << std::endl;
      }
   }
   catch(std::exception& e)
      {
      printf("Exception caught: %s\n", e.what());
      }
   return 0;
   }
