/*
Written by Jack Lloyd (lloyd@acm.jhu.edu), on Prickle-Prickle,
the 10th of Bureaucracy, 3167.

This file is in the public domain

This is just like the normal MD5 application, but uses the Unix I/O system
calls instead of C++ iostreams. Previously, this version was much faster and
smaller, but GCC 3.1's libstdc++ seems to have been improved enough that the
difference is now fairly minimal (10-20%)

Nicely enough, doing the change required changing only about 3 lines of code.

Note that this requires you to be on a machine running some sort of Unix. Well,
I guess any POSIX.1 compliant OS (in theory).
*/

#include <iostream>
#include <opencl/md5.h>
#include <opencl/hex.h>
#include <opencl/filters.h>

#ifndef OPENCL_EXT_PIPE_UNIXFD_IO
#error "You didn't compile the pipe_unixfd module into OpenCL"
#endif

#include <fcntl.h>
#include <unistd.h>

int main(int argc, char* argv[])
   {
   if(argc < 2)
      {
      std::cout << "Usage: md5_fd <filenames>" << std::endl;
      return 1;
      }

   OpenCL::LibraryInitializer init;

   OpenCL::Pipe pipe(new OpenCL::Hash_Filter(new OpenCL::MD5),
                     new OpenCL::Hex_Encoder);

   int skipped = 0;
   for(int j = 1; argv[j] != 0; j++)
      {
      int file = open(argv[j], O_RDONLY);
      if(file == -1)
         {
         std::cout << "ERROR: could not open " << argv[j] << std::endl;
         skipped++;
         continue;
         }
      pipe.start_msg();
      file >> pipe;
      pipe.end_msg();
      close(file);
      pipe.set_default_msg(j-1-skipped);
      std::cout << pipe << "  " << argv[j] << std::endl;
      }
   return 0;
   }
