/*************************************************
* Diffie-Hellman Header File                     *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_DIFFIE_HELLMAN_H__
#define OPENCL_DIFFIE_HELLMAN_H__

#include <opencl/bigint.h>
#include <opencl/pk_keys.h>
#include <opencl/mod_exp.h>
#include <opencl/dl_parm.h>

namespace OpenCL {

/*************************************************
* Diffie-Hellman Public Key                      *
*************************************************/
class DH_PublicKey
   {
   public:
      virtual bool check_params() const;
      const BigInt& get_p() const { return p; }
      const BigInt& get_g() const { return g; }
      const BigInt& get_y() const { return y; }

      DH_PublicKey(const DL_Group&, const BigInt&);
      DH_PublicKey(const DH_PublicKey&);
      virtual ~DH_PublicKey() {}
   protected:
      DL_Group params;
      const BigInt &p, &g;
      BigInt y;
   };

/*************************************************
* Diffie-Hellman Private Key                     *
*************************************************/
class DH_PrivateKey : public DH_PublicKey,
                      public PK_Key_Agreement_Key
   {
   public:
      SecureVector<byte> derive_key(const byte[], u32bit) const;
      SecureVector<byte> derive_key(const DH_PublicKey&) const;
      SecureVector<byte> derive_key(const BigInt&) const;

      bool check_params() const;
      const BigInt& get_x() const { return x; }

      DH_PrivateKey(const DL_Group&);
      DH_PrivateKey(const DL_Group&, const BigInt&, const BigInt& = 0);
   private:
      BigInt x;
      FixedExponent_Exp powermod_x_p;
   };

}

#endif
