/*************************************************
* Discrete Logarithm Group Header File           *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_DL_PARM_H__
#define OPENCL_DL_PARM_H__

#include <opencl/bigint.h>

namespace OpenCL {

/*************************************************
* Discrete Logarithm Group                       *
*************************************************/
class DL_Group
   {
   public:
      const BigInt& get_p() const;
      const BigInt& get_q() const;
      const BigInt& get_g() const;

      enum PrimeType { Strong, Prime_Subgroup, DSA_Kosherizer };

      bool prime_is_safe() const;
      bool verify() const;

      DL_Group();
      DL_Group(u32bit, PrimeType = Strong);
      DL_Group(const byte[], u32bit, u32bit = 1024, u32bit = 0);
      DL_Group(const BigInt&, const BigInt&);
      DL_Group(const BigInt&, const BigInt&, const BigInt&);
   private:
      BigInt p, q, g;
      bool safe_prime, initialized;
   };

/*************************************************
* Retrieve a DL group by name                    *
*************************************************/
const DL_Group& get_dl_group(const std::string&);

}

#endif
