/*************************************************
* ECB Mode Header File                           *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_ECB_H__
#define OPENCL_ECB_H__

#include <opencl/modebase.h>

namespace OpenCL {

/*************************************************
* ECB Base Class                                 *
*************************************************/
class ECB : public BlockCipherMode
   {
   public:
      void write(const byte[], u32bit);
      void end_msg();
      ECB(BlockCipher*, const BlockCipher_Key&, const std::string&);
      ECB(const std::string&, const BlockCipher_Key&, const std::string&);
   private:
      virtual void process_block(const byte[], byte[]) = 0;
   };

/*************************************************
* ECB Encryption                                 *
*************************************************/
class ECB_Encryption : public ECB
   {
   public:
      ECB_Encryption(BlockCipher* c, const BlockCipher_Key& key) :
         ECB(c, key, "ECB_Encryption") {}
      ECB_Encryption(const std::string&, const BlockCipher_Key&);
   private:
      void process_block(const byte[], byte[]);
   };

/*************************************************
* ECB Decryption                                 *
*************************************************/
class ECB_Decryption : public ECB
   {
   public:
      ECB_Decryption(BlockCipher* c, const BlockCipher_Key& key) :
         ECB(c, key, "ECB_Decryption") {}
      ECB_Decryption(const std::string&, const BlockCipher_Key&);
   private:
      void process_block(const byte[], byte[]);
   };

}

#endif
