/*************************************************
* Filters Header File                            *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_FILTERS_H__
#define OPENCL_FILTERS_H__

#include <opencl/filter.h>
#include <opencl/key_filt.h>
#include <opencl/fork.h>
#include <opencl/chain.h>
#include <opencl/symkey.h>
#include <opencl/pipe.h>

namespace OpenCL {

/*************************************************
* Stream Cipher Filter                           *
*************************************************/
class StreamCipher_Filter : public Keyed_Filter
   {
   public:
      void reset_key(const StreamCipher_Key& key) { cipher->set_key(key); }
      void write(const byte[], u32bit);

      StreamCipher_Filter(StreamCipher*, const StreamCipher_Key&);
      StreamCipher_Filter(const std::string&, const StreamCipher_Key&);
      ~StreamCipher_Filter() { delete cipher; }
   private:
      StreamCipher* cipher;
      SecureBuffer<byte, DEFAULT_BUFFERSIZE> buffer;
   };

/*************************************************
* Hash Filter                                    *
*************************************************/
class Hash_Filter : public Filter
   {
   public:
      void write(const byte input[], u32bit len) { hash->update(input, len); }
      void end_msg();

      Hash_Filter(HashFunction*, u32bit = 0);
      Hash_Filter(const std::string&, u32bit = 0);
      ~Hash_Filter() { delete hash; }
   private:
      const u32bit OUTPUT_LENGTH;
      HashFunction* hash;
   };

/*************************************************
* MessageAuthenticationCode Filter               *
*************************************************/
class MAC_Filter : public Keyed_Filter
   {
   public:
      void reset_key(const MAC_Key& key) { mac->set_key(key); }
      void write(const byte input[], u32bit len) { mac->update(input, len); }
      void end_msg();

      MAC_Filter(MessageAuthenticationCode*, const MAC_Key&, u32bit = 0);
      MAC_Filter(const std::string&, const MAC_Key&, u32bit = 0);
      ~MAC_Filter() { delete mac; }
   private:
      const u32bit OUTPUT_LENGTH;
      MessageAuthenticationCode* mac;
   };

}

#endif
