/*************************************************
* HAVAL Header File                              *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_HAVAL_H__
#define OPENCL_HAVAL_H__

#include <opencl/mdx_hash.h>

namespace OpenCL {

/*************************************************
* HAVAL Base Class                               *
*************************************************/
class HAVAL : public MDx_HashFunction
   {
   public:
      static const u32bit VERSION = 1, PASS = 5;
      void clear() throw();
      std::string name() const;
      HashFunction* clone() const { return new HAVAL(OUTPUT_LENGTH); }
   protected:
      HAVAL(u32bit hashlen) : MDx_HashFunction(hashlen, 128) { clear(); }
   private:
      void final_result(byte[]);
      void hash(const byte[]);
      void tailor_digest(u32bit);
      static void H1(u32bit&, u32bit, u32bit, u32bit, u32bit,
                     u32bit,  u32bit, u32bit, u32bit, u32bit);
      static void H2(u32bit&, u32bit, u32bit, u32bit, u32bit,
                     u32bit,  u32bit, u32bit, u32bit, u32bit);
      static void H3(u32bit&, u32bit, u32bit, u32bit, u32bit,
                     u32bit,  u32bit, u32bit, u32bit, u32bit);
      static void H4(u32bit&, u32bit, u32bit, u32bit, u32bit,
                     u32bit,  u32bit, u32bit, u32bit, u32bit);
      static void H5(u32bit&, u32bit, u32bit, u32bit, u32bit,
                     u32bit,  u32bit, u32bit, u32bit, u32bit);
      SecureBuffer<u32bit, 32> M;
      SecureBuffer<u32bit, 8> digest;
   };

/*************************************************
* HAVAL-128                                      *
*************************************************/
struct HAVAL_128 : public HAVAL
   {
   HAVAL_128() : HAVAL(16) {}
   };

/*************************************************
* HAVAL-160                                      *
*************************************************/
struct HAVAL_160 : public HAVAL
   {
   HAVAL_160() : HAVAL(20) {}
   };

/*************************************************
* HAVAL-192                                      *
*************************************************/
struct HAVAL_192 : public HAVAL
   {
   HAVAL_192() : HAVAL(24) {}
   };

/*************************************************
* HAVAL-224                                      *
*************************************************/
struct HAVAL_224 : public HAVAL
   {
   HAVAL_224() : HAVAL(28) {}
   };

/*************************************************
* HAVAL-256                                      *
*************************************************/
struct HAVAL_256 : public HAVAL
   {
   HAVAL_256() : HAVAL(32) {}
   };

}

#endif

