/*************************************************
* Modular Exponentiation Header File             *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_MODULAR_EXP_H__
#define OPENCL_MODULAR_EXP_H__

#include <opencl/bigint.h>
#include <opencl/reducer.h>

namespace OpenCL {

/*************************************************
* Fixed Exponent Exponentiation                  *
*************************************************/
class FixedExponent_Exp
   {
   public:
      BigInt operator() (const BigInt& n) const { return power_mod(n); }
      BigInt reduce(const BigInt& n) const { return reducer->reduce(n); }
      BigInt power_mod(const BigInt&) const;

      const BigInt& get_exponent() const { return exponent; }
      const BigInt& get_modulus() const { return reducer->get_modulus(); }

      FixedExponent_Exp& operator=(const FixedExponent_Exp&);

      FixedExponent_Exp() { reducer = 0; }
      FixedExponent_Exp(const BigInt&, const BigInt&);
      FixedExponent_Exp(const FixedExponent_Exp&);
      ~FixedExponent_Exp() { delete reducer; }
   private:
      ModularReducer* reducer;
      BigInt exponent;
   };

/*************************************************
* Fixed Base Exponentiation                      *
*************************************************/
class FixedBase_Exp
   {
   public:
      BigInt operator() (const BigInt& n) const { return power_mod(n); }
      BigInt reduce(const BigInt& n) const { return reducer->reduce(n); }
      BigInt power_mod(const BigInt&) const;

      const BigInt& get_base() const { return base; }
      const BigInt& get_modulus() const { return reducer->get_modulus(); }

      FixedBase_Exp& operator=(const FixedBase_Exp&);

      FixedBase_Exp() { reducer = 0; }
      FixedBase_Exp(const BigInt&, const BigInt&);
      FixedBase_Exp(const FixedBase_Exp&);
      ~FixedBase_Exp() { delete reducer; }
   private:
      ModularReducer* reducer;
      BigInt base;
   };

}

#endif
