/*************************************************
* Miscellaneous PK Functions Header File         *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_PUBKEY_MISC_H__
#define OPENCL_PUBKEY_MISC_H__

#include <opencl/base.h>

namespace OpenCL {

/*************************************************
* Encoding Method for Encryption                 *
*************************************************/
class EME
   {
   public:
      virtual u32bit maximum_input_size(u32bit) const = 0;
      SecureVector<byte> encode(const byte[], u32bit, u32bit) const;
      SecureVector<byte> encode(const SecureVector<byte>&, u32bit) const;
      SecureVector<byte> decode(const byte[], u32bit, u32bit) const;
      SecureVector<byte> decode(const SecureVector<byte>&, u32bit) const;

      virtual ~EME() {}
   private:
      virtual SecureVector<byte> pad(const byte[], u32bit, u32bit) const = 0;
      virtual SecureVector<byte> unpad(const byte[], u32bit, u32bit) const = 0;
   };

/*************************************************
* Encoding Method for Signatures, Appendix       *
*************************************************/
class EMSA
   {
   public:
      virtual void update(const byte[], u32bit) = 0;
      virtual SecureVector<byte> raw_data() = 0;
      virtual SecureVector<byte> encoding_of(const byte[], u32bit, u32bit) = 0;

      SecureVector<byte> encoding(u32bit);

      bool is_valid_encoding(const byte[], u32bit,
                             const byte[], u32bit, u32bit) throw();
      bool is_valid_encoding(const SecureVector<byte>&,
                             const SecureVector<byte>&, u32bit) throw();
      virtual ~EMSA() {}
   private:
      virtual bool verify(const byte[], u32bit,
                          const byte[], u32bit, u32bit);
   };

/*************************************************
* Mask Generation Function                       *
*************************************************/
class MGF
   {
   public:
      virtual void mask(const byte[], u32bit, byte[], u32bit) const = 0;
      virtual ~MGF() {}
   };

/*************************************************
* Key Derivation Function                        *
*************************************************/
class KDF
   {
   public:
      SymmetricKey derive_key(const byte[], u32bit, u32bit) const;
      SymmetricKey derive_key(const SecureVector<byte>&, u32bit) const;
      virtual ~KDF() {}
   private:
      virtual SymmetricKey derive(const byte[], u32bit, u32bit) const = 0;
   };

}

#endif
