/*************************************************
* Public Key Types Header File                   *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_PK_TYPES_H__
#define OPENCL_PK_TYPES_H__

#include <opencl/pubkey.h>
#include <opencl/ieee1363.h>
#include <opencl/pkcs1.h>

#include <opencl/dsa.h>
#include <opencl/rsa.h>
#include <opencl/rw.h>

#include <opencl/sha1.h>
#include <opencl/md5.h>

#ifdef __GNUG__
 #warning pk_types.h is deprecated, use the functions in look_pk.h instead
#endif

/*************************************************
* RSA Encryption Schemes                         *
*************************************************/
#define RSA_EME1_SHA1_Encryptor(key) \
  PK_Encryptor(key, new OpenCL::EME1("SHA-1"))
#define RSA_EME1_SHA1_Decryptor(key) \
  PK_Decryptor(key, new OpenCL::EME1("SHA-1"))

#define RSA_PKCS1v15_Encryptor(key) \
  OpenCL::PK_Encryptor(key, new OpenCL::EME_PKCS1)
#define RSA_PKCS1v15_Decryptor(key) \
  OpenCL::PK_Decryptor(key, new OpenCL::EME_PKCS1)

#define RSA_Raw_Encryptor(key) OpenCL::PK_Encryptor(key)
#define RSA_Raw_Decryptor(key) OpenCL::PK_Decryptor(key)

/*************************************************
* RSA Signature Schemes                          *
*************************************************/
#define IFSSA_EMSA_Verifier(key, hash, pad) \
  OpenCL::PK_Verifier_with_MR(key, new OpenCL::hash, \
                              new OpenCL::pad(new OpenCL::hash))
#define IFSSA_EMSA_Signer(key, hash, pad) \
  OpenCL::PK_Signer(key, new OpenCL::hash, new OpenCL::pad(new OpenCL::hash))

#define RSA_Raw_SHA1_Verifier(key) \
  OpenCL::PK_Verifier_with_MR(key, new OpenCL::SHA1)
#define RSA_Raw_SHA1_Signer(key)   OpenCL::PK_Signer(key, new OpenCL::SHA1)

#define RSA_EMSA2_SHA1_Verifier(key) IFSSA_EMSA_Verifier(key, SHA1, EMSA2)
#define RSA_EMSA2_SHA1_Signer(key)   IFSSA_EMSA_Signer(key, SHA1, EMSA2)

#define RSA_EMSA3_SHA1_Verifier(key) IFSSA_EMSA_Verifier(key, SHA1, EMSA3)
#define RSA_EMSA3_SHA1_Signer(key)   IFSSA_EMSA_Signer(key, SHA1, EMSA3)

#define RSA_EMSA3_MD5_Verifier(key)  IFSSA_EMSA_Verifier(key, MD5, EMSA3)
#define RSA_EMSA3_MD5_Signer(key)    IFSSA_EMSA_Signer(key, MD5, EMSA3)

/*************************************************
* DSA Signature Schemes                          *
*************************************************/
#define DSA_Verifier(key) OpenCL::PK_Verifier_wo_MR(key, new OpenCL::SHA1)
#define DSA_Signer(key)   OpenCL::PK_Signer(key, new OpenCL::SHA1)

/*************************************************
* Rabin-Williams Signature Schemes               *
*************************************************/
#define RW_EMSA2_SHA1_Verifier(key) IFSSA_EMSA_Verifier(key, SHA1, EMSA2)
#define RW_EMSA2_SHA1_Signer(key)   IFSSA_EMSA_Signer(key, SHA1, EMSA2)

#endif
