/*************************************************
* Discrete Logarithm Parameters Source File      *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/dl_parm.h>
#include <cstring>
#include <map>

namespace OpenCL {

namespace {

std::map<std::string, DL_Group> dl_groups;

/*************************************************
* Add a single group to the map                  *
*************************************************/
void add_group(const std::string& name, const char* g, const char* p)
   {
   dl_groups[name] =
      DL_Group(BigInt((const byte*)p, std::strlen(p), Hexadecimal),
               BigInt((const byte*)g, std::strlen(g), Hexadecimal));
   }

/*************************************************
* Retrieve a DL domain paramter set by name      *
*************************************************/
void load_dl_groups()
   {
   add_group("IETF-1024", "2",
             "FFFFFFFFFFFFFFFFC90FDAA22168C234C4C6628B80DC1CD1"
             "29024E088A67CC74020BBEA63B139B22514A08798E3404DD"
             "EF9519B3CD3A431B302B0A6DF25F14374FE1356D6D51C245"
             "E485B576625E7EC6F44C42E9A637ED6B0BFF5CB6F406B7ED"
             "EE386BFB5A899FA5AE9F24117C4B1FE649286651ECE65381"
             "FFFFFFFFFFFFFFFF");
   }

}

/*************************************************
* Retrieve a DL domain paramter set by name      *
*************************************************/
const DL_Group& get_dl_group(const std::string& name)
   {
   static bool initialized = false;
   if(!initialized)
      load_dl_groups();

   std::map<std::string, DL_Group>::const_iterator group;
   group = dl_groups.find(name);
   if(group != dl_groups.end())
      return group->second;

   throw Exception("get_dl_paramaters: Domain parameters " + name +
                   " not found");

   }

/*************************************************
* Delete the groups                              *
*************************************************/
void destroy_dl_groups()
   {
   dl_groups.clear();
   }

}
