/*************************************************
* Filters Source File                            *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/filters.h>
#include <opencl/lookup.h>

#include <stdio.h>

namespace OpenCL {

/*************************************************
* StreamCipher_Filter Constructor                *
*************************************************/
StreamCipher_Filter::StreamCipher_Filter(StreamCipher* sc,
                                         const StreamCipher_Key& key)
   {
   cipher = sc;
   cipher->set_key(key);
   }

/*************************************************
* StreamCipher_Filter Constructor                *
*************************************************/
StreamCipher_Filter::StreamCipher_Filter(const std::string& sc_name,
                                         const StreamCipher_Key& key)
   {
   cipher = get_stream_cipher(sc_name);
   cipher->set_key(key);
   }

/*************************************************
* Write data into a StreamCipher_Filter          *
*************************************************/
void StreamCipher_Filter::write(const byte input[], u32bit length)
   {
   while(length)
      {
      u32bit copied = std::min(length, buffer.size());
      cipher->encrypt(input, buffer, copied);
      send(buffer, copied);
      input += copied;
      length -= copied;
      }
   }

/*************************************************
* Hash_Filter Constructor                        *
*************************************************/
Hash_Filter::Hash_Filter(HashFunction* h, u32bit len) : OUTPUT_LENGTH(len)
   {
   printf("WARNING: This constructor of Hash_Filter is deprecated; "
          "(read doc/deprecated.txt for details)\n");
   hash = h;
   }

/*************************************************
* Hash_Filter Constructor                        *
*************************************************/
Hash_Filter::Hash_Filter(const std::string& hash_name, u32bit len) :
   OUTPUT_LENGTH(len)
   {
   hash = get_hash(hash_name);
   }

/*************************************************
* Complete a calculation by a Hash_Filter        *
*************************************************/
void Hash_Filter::end_msg()
   {
   SecureVector<byte> output = hash->final();
   if(OUTPUT_LENGTH)
      send(output, std::min(OUTPUT_LENGTH, output.size()));
   else
      send(output, output.size());
   }

/*************************************************
* MAC_Filter Constructor                         *
*************************************************/
MAC_Filter::MAC_Filter(MessageAuthenticationCode* m, const MAC_Key& key,
                       u32bit len) : OUTPUT_LENGTH(len)
   {
   mac = m;
   mac->set_key(key);
   }

/*************************************************
* MAC_Filter Constructor                         *
*************************************************/
MAC_Filter::MAC_Filter(const std::string& mac_name, const MAC_Key& key,
                       u32bit len) : OUTPUT_LENGTH(len)
   {
   mac = get_mac(mac_name);
   mac->set_key(key);
   }

/*************************************************
* Complete a calculation by a MAC_Filter         *
*************************************************/
void MAC_Filter::end_msg()
   {
   SecureVector<byte> output = mac->final();
   if(OUTPUT_LENGTH)
      send(output, std::min(OUTPUT_LENGTH, output.size()));
   else
      send(output, output.size());
   }

}
