/*************************************************
* Encoding Algorithm Retrieval Source File       *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/look_pk.h>
#include <opencl/lookup.h>
#include <opencl/emsa1.h>
#include <opencl/emsa2.h>
#include <opencl/emsa3.h>
#include <opencl/emsa_raw.h>
#include <opencl/eme1.h>
#include <opencl/pkcs1.h>

namespace OpenCL {

/*************************************************
* Get an EMSA by name                            *
*************************************************/
EMSA* get_emsa(const std::string& emsa)
   {
   if(emsa == "Raw")
      return new EMSA_Raw;

   std::vector<std::string> name = parse_algorithm_name(emsa);
   if(name.size() != 2)
      throw Invalid_Argument("Invalid EMSA name: " + emsa);

   const std::string emsa_name = deref_alias(name[0]);
   const std::string hash_name = name[1];

   if(emsa_name == "EMSA1")
      return new EMSA1(hash_name);
   if(emsa_name == "EMSA2")
      return new EMSA2(hash_name);
   if(emsa_name == "EMSA3")
      return new EMSA3(hash_name);
   throw Algorithm_Not_Found(emsa);
   }

/*************************************************
* Get an EME by name                             *
*************************************************/
EME* get_eme(const std::string& eme)
   {
   if(eme == "PKCS1v15")
      return new EME_PKCS1;

   std::vector<std::string> name = parse_algorithm_name(eme);
   if(name.size() != 2)
      throw Invalid_Argument("Invalid EME name: " + eme);

   const std::string eme_name = deref_alias(name[0]);
   const std::string hash_name = name[1];

   if(eme_name == "EME1")
      return new EME1(hash_name);
   throw Algorithm_Not_Found(eme);
   }

}
