/*************************************************
* Modular Exponentiation Source File             *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/mod_exp.h>
#include <opencl/reducer.h>
#include <opencl/numthry.h>

namespace OpenCL {

/*************************************************
* Fixed Exponent Exponentiation                  *
*************************************************/
BigInt FixedExponent_Exp::power_mod(const BigInt& base) const
   {
   return OpenCL::power_mod(reducer->reduce(base), exponent, reducer);
   }

/*************************************************
* FixedExponent_Exp Assignment Operator          *
*************************************************/
FixedExponent_Exp& FixedExponent_Exp::operator=(const FixedExponent_Exp& exp)
   {
   reducer = get_reducer(exp.get_modulus());
   exponent = exp.get_exponent();
   return (*this);
   }

/*************************************************
* FixedExponent_Exp Constructor                  *
*************************************************/
FixedExponent_Exp::FixedExponent_Exp(const BigInt& exp, const BigInt& mod) :
   reducer(get_reducer(mod)), exponent(exp)
   {
   if(get_modulus() <= 0)
      throw Invalid_Argument("FixedExponent_Exp: Invalid modulus");
   if(get_exponent() < 0)
      throw Invalid_Argument("FixedExponent_Exp: Invalid exponent");
   }

/*************************************************
* FixedExponent_Exp Copy Constructor             *
*************************************************/
FixedExponent_Exp::FixedExponent_Exp(const FixedExponent_Exp& exp)
   {
   exponent = exp.get_exponent();
   reducer = get_reducer(exp.get_modulus());
   }

/*************************************************
* Fixed Base Exponentiation                      *
*************************************************/
BigInt FixedBase_Exp::power_mod(const BigInt& exponent) const
   {
   return OpenCL::power_mod(base, exponent, reducer);
   }

/*************************************************
* FixedBase_Exp Assignment Operator              *
*************************************************/
FixedBase_Exp& FixedBase_Exp::operator=(const FixedBase_Exp& exp)
   {
   reducer = get_reducer(exp.get_modulus());
   base = exp.get_base();
   return (*this);
   }

/*************************************************
* FixedBase_Exp Constructor                      *
*************************************************/
FixedBase_Exp::FixedBase_Exp(const BigInt& exp, const BigInt& mod) :
   reducer(get_reducer(mod)), base(exp)
   {
   if(get_modulus() <= 0)
      throw Invalid_Argument("FixedBase_Exp: Invalid modulus");
   if(get_base() < 0)
      throw Invalid_Argument("FixedBase_Exp: Invalid base");
   }

/*************************************************
* FixedBase_Exp Copy Constructor                 *
*************************************************/
FixedBase_Exp::FixedBase_Exp(const FixedBase_Exp& exp) :
   reducer(get_reducer(exp.get_modulus())), base(exp.get_base())
   {
   }

}
