/*************************************************
* Three Operand MP Algorithms Source File        *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/mp_core.h>
#include <string.h>

extern "C" {

/*************************************************
* Three Operand Addition                         *
*************************************************/
void bigint_add3(word* const z,
                 const word* x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   if(x_size < y_size) { bigint_add3(z, y, y_size, x, x_size); return; }

   word carry = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      dword sum = ((dword)x[j] + y[j]) + carry;
      z[j] = (sum & MP_WORD_MAX);
      carry = (sum >> MP_WORD_BITS) & MP_WORD_MAX;
      }

   for(u32bit j = y_size; j != x_size; j++)
      {
      if(carry == 0)
         {
         memcpy(z + j, x + j, (x_size - j) * sizeof(word));
         break;
         }
      z[j] = x[j] + carry;
      carry = (carry && x[j] == MP_WORD_MAX) ? 1 : 0;
      }
   z[x_size] = carry;
   }

/*************************************************
* Three Operand Subtraction                      *
*************************************************/
void bigint_sub3(word* const z,
                 const word* x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   word borrow = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      dword temp = ((MP_RADIX + x[j]) - y[j]) - borrow;
      borrow = ((temp < MP_RADIX) ? 1 : 0);
      z[j] = (word)(temp - MP_RADIX);
      }
   for(u32bit j = y_size; j != x_size; j++)
      {
      dword temp = (MP_RADIX + x[j]) - borrow;
      borrow = ((temp < MP_RADIX) ? 1 : 0);
      z[j] = (word)(temp - MP_RADIX);
      }
   }

/*************************************************
* Three Operand Multiplication                   *
*************************************************/
void bigint_mul3(word* const z,
                 const word* x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   for(u32bit j = 0; j != x_size; j++)
      {
      word carry = 0;
      for(u32bit k = 0; k != y_size; k++)
         {
         dword product = (dword)z[j+k] + (dword)x[j] * y[k] + carry;
         z[j+k] = (product & MP_WORD_MAX);
         carry = (product >> MP_WORD_BITS) & MP_WORD_MAX;
         }
      z[j+y_size] = carry;
      }
   }

/*************************************************
* Three Operand Linear Multiply                  *
*************************************************/
void bigint_linmul3(word* const z,
                    const word* x, u32bit x_size, word y)
   {
   word carry = 0;
   for(u32bit j = 0; j != x_size; j++)
      {
      dword product = (dword)x[j] * y + carry;
      z[j] = (product & MP_WORD_MAX);
      carry = (product >> MP_WORD_BITS) & MP_WORD_MAX;
      }
   z[x_size] = carry;
   }

}
