/*************************************************
* OFB Mode Source File                           *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/ofb.h>

namespace OpenCL {

/*************************************************
* OFB Constructor                                *
*************************************************/
OFB::OFB(BlockCipher* c, const BlockCipher_Key& key,
         const BlockCipherModeIV& iv) :
   BlockCipherMode(c, "OFB", key, iv)
   {
   cipher->encrypt(state);
   }

/*************************************************
* OFB Constructor                                *
*************************************************/
OFB::OFB(const std::string& cipher_name, const BlockCipher_Key& key,
         const BlockCipherModeIV& iv) :
   BlockCipherMode(cipher_name, "OFB", key, iv)
   {
   cipher->encrypt(state);
   }

/*************************************************
* OFB Encryption/Decryption                      *
*************************************************/
void OFB::write(const byte input[], u32bit length)
   {
   while(length)
      {
      u32bit copied = std::min(BLOCK_SIZE - position, length);
      xor_buf(buffer, input, state + position, copied);
      send(buffer, copied);
      input += copied;
      length -= copied;
      position += copied;
      if(position == BLOCK_SIZE)
         {
         cipher->encrypt(state);
         position = 0;
         }
      }
   }

/*************************************************
* Counter Constructor                            *
*************************************************/
Counter::Counter(BlockCipher* c, const BlockCipher_Key& key,
                 const BlockCipherModeIV& iv) :
   BlockCipherMode(c, "Counter", key, iv)
   {
   cipher->encrypt(state, buffer);
   }

/*************************************************
* Counter Constructor                            *
*************************************************/
Counter::Counter(const std::string& cipher_name, const BlockCipher_Key& key,
                 const BlockCipherModeIV& iv) :
   BlockCipherMode(cipher_name, "Counter", key, iv)
   {
   cipher->encrypt(state, buffer);
   }

/*************************************************
* Counter Encryption/Decryption                  *
*************************************************/
void Counter::write(const byte input[], u32bit length)
   {
   while(length)
      {
      u32bit copied = std::min(BLOCK_SIZE - position, length);
      xor_buf(buffer + position, input, copied);
      send(buffer + position, copied);
      input += copied;
      length -= copied;
      position += copied;
      if(position == BLOCK_SIZE)
         {
         for(s32bit j = BLOCK_SIZE - 1; j >= 0; j--)
            if(++state[j])
               break;
         cipher->encrypt(state, buffer);
         position = 0;
         }
      }
   }

}
